@@ -1,11 +1,12 @@
+from __future__ import absolute_import
+
 import blinker
 from collections import deque
 from functools import wraps, partial
 from threading import local
-import inspect
 import sys
 
-from .compat import reraise, itervalues, iteritems, is_nextable
+from .compat import reraise, iteritems, is_nextable
 
 def noop(*_, **dummy):
     pass
@@ -46,11 +47,8 @@ class _PendingRunnable(object):
 
         try:
             requirements = run_fn()
-        except StopIterationWithValue as e:
-            self.result = e.value
-            self.iteration = -1
-            return None
-        except StopIteration:
+        except StopIteration as e:
+            self.result = getattr(e, 'value', None)
             self.iteration = -1
             return None
         except Exception:
@@ -75,9 +73,6 @@ class _PendingRunnable(object):
             self.dependency_results = None
             self.dependency_completed = partial(self._dependency_completed_single, self.iteration)
 
-        assert all(is_nextable(dep) for dep in itervalues(dependencies)), \
-            inspect.getframeinfo(self.iterable.gi_frame)
-
         self.dependency_threw = partial(self._dependency_threw, self.iteration)
         self.dependencies_remaining = len(dependencies)
         return dependencies
@@ -193,29 +188,67 @@ class RunLoop(object):
             if runnable.ready:
                 self.run_queue.append(runnable)
 
-class _LocalRunLoop(local):
+class _ThreadingLocalRunLoop(local):
     loop = None
 
-_CURRENT_RUN_LOOP = _LocalRunLoop()
+_CURRENT_RUN_LOOP = _ThreadingLocalRunLoop()
 
 def current_run_loop():
     return _CURRENT_RUN_LOOP.loop
 
+def use_threading_local():
+    assert current_run_loop() is None
+
+    global _CURRENT_RUN_LOOP
+    _CURRENT_RUN_LOOP = _ThreadingLocalRunLoop()
+
+try:
+    from gevent.local import local as gevent_local
+except ImportError as ex:
+    def use_gevent_local():
+        raise ImportError("Gevent not present")
+else:
+    class _GeventLocalRunLoop(gevent_local):
+        loop = None
+
+    def use_gevent_local():
+        assert current_run_loop() is None
+
+        global _CURRENT_RUN_LOOP
+        _CURRENT_RUN_LOOP = _GeventLocalRunLoop()
+
 def runloop_coroutine():
+    """Creates a coroutine that gets run in a run loop.
+
+    The run loop will be created if necessary."""
     def wrap(fn):
         @wraps(fn)
         def wrapper(*args, **kwargs):
             if _CURRENT_RUN_LOOP.loop:
-                return fn(*args, **kwargs)  # Returns the underlying iterable.
+                it = fn(*args, **kwargs)
+                assert is_nextable(it), '%s did not return an iterator' % (fn)
+                return it
             else:
                 _CURRENT_RUN_LOOP.loop = loop = RunLoop()
                 try:
-                    return loop.run(fn(*args, **kwargs))
+                    it = fn(*args, **kwargs)
+                    assert is_nextable(it), '%s did not return an iterator' % (fn)
+                    return loop.run(it)
                 finally:
                     _CURRENT_RUN_LOOP.loop = None
         return wrapper
     return wrap
 
+def requires_runloop():
+    """Same as @runloop_coroutine, but refuses to create a loop if one is not present."""
+    def wrap(fn):
+        @wraps(fn)
+        def wrapper(*args, **kwargs):
+            assert current_run_loop()
+            return fn(*args, **kwargs)
+        return wrapper
+    return wrap
+
 def coro_return(value):
     raise StopIterationWithValue(value)
 
@@ -226,6 +259,7 @@ class _DeferredIterable(object):
         self.ready = False
         self.batch_context = None
         self.runnable = None
+        self.on_ready = blinker.Signal()
 
     def on_add_to_loop(self, context, runnable):
         assert self.batch_context is None
@@ -234,33 +268,45 @@ class _DeferredIterable(object):
         self.runnable = runnable
 
     def set_value(self, value):
-        self.value = value
+        assert not self.ready
         self.ready = True
+
+        self.value = value
         if self.batch_context:
             self.batch_context.runnable(self.runnable)
 
+        self.on_ready.send()
+
     def set_exception(self, type_, value=None, traceback=None):
-        self.exception = (type_, value, traceback)
+        assert not self.ready
         self.ready = True
+
+        self.exception = (type_, value, traceback)
         if self.batch_context:
             self.batch_context.runnable(self.runnable)
 
+        self.on_ready.send()
+
     def __next__(self):
         coro_return(self.get())
     next = __next__
 
     def get(self):
+        if __debug__:
+            if not self.ready:
+                raise ValueError(".get() on non-ready deferred.")
         if self.exception is not None:
             reraise(*self.exception)
         return self.value
 
-
+@requires_runloop()
 def deferred():
     assert current_run_loop()
     coro_return(_DeferredIterable())
     yield  # pragma: no cover
 
 
+@requires_runloop()
 def future(iterable):
     """Given an iterable, this returns an object that can be yielded again once
     you want to use it's value. This is useful to "front-load" some expensive
@@ -285,3 +331,41 @@ def future(iterable):
     result = yield deferred()
     current_run_loop().add(iterable, result.set_value, result.set_exception)
     coro_return(result)
+
+@requires_runloop()
+def wait(deferreds, count=None):
+    """iwait(deferreds_or_futures, count=None).
+
+    Waits until up to `count` (or all, if count is None) deferreds to complete. Returns
+    the objects that completed. Example:
+
+    a, b, c = yield future(get_a()), future(get_b()), future(get_c())
+    first, second = yield wait([a, b, c], count=2)
+    # At this point 2/3 of the above futures are complete."""
+    if count is None:
+        count = len(deferreds)
+
+    assert count <= len(deferreds), 'Waiting on too many deferreds: %s' % (count)
+
+    ready_list = [d for d in deferreds if d.ready]
+    # Check if any of the deferreds are ready.
+
+    if len(ready_list) < count:
+        wait_deferred = yield deferred()
+
+        for d in deferreds:
+            def on_ready(_):
+                if wait_deferred.ready:
+                    return  # This is mostly necessary for PyPy because weak refs
+                            # aren't immediately removed there.
+
+                ready_list.append(d)
+                if len(ready_list) >= count:
+                    wait_deferred.set_value(True)
+
+            d.on_ready.connect(on_ready, weak=True)
+
+        yield wait_deferred
+
+    assert len(ready_list) == count
+    coro_return(ready_list)
